"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = __importStar(require("react"));
var react_dom_1 = __importStar(require("react-dom"));
var getDisplayName_1 = __importDefault(require("./getDisplayName"));
var utils_1 = require("./utils");
var rootId = 'cypress-root';
function checkMountModeEnabled() {
    // @ts-ignore
    if (Cypress.spec.specType !== 'component') {
        throw new Error("In order to use mount or unmount functions please place the spec in component folder");
    }
}
/**
 * Inject custom style text or CSS file or 3rd party style resources
 */
var injectStyles = function (options) { return function () {
    var document = cy.state('document');
    var el = document.getElementById(rootId);
    return utils_1.injectStylesBeforeElement(options, document, el);
}; };
/**
 * Mount a React component in a blank document; register it as an alias
 * To access: use an alias or original component reference
 *  @function   cy.mount
 *  @param      {React.ReactElement}  jsx - component to mount
 *  @param      {string}  [Component] - alias to use later
 *  @example
 ```
 import Hello from './hello.jsx'
 // mount and access by alias
 cy.mount(<Hello />, 'Hello')
 // using default alias
 cy.get('@Component')
 // using original component
 cy.get(Hello)
 ```
 **/
exports.mount = function (jsx, options) {
    if (options === void 0) { options = {}; }
    checkMountModeEnabled();
    // Get the display name property via the component constructor
    // @ts-ignore FIXME
    var displayName = getDisplayName_1.default(jsx.type, options.alias);
    var logInstance;
    return cy
        .then(function () {
        if (options.log !== false) {
            logInstance = Cypress.log({
                name: 'mount',
                message: ["ReactDOM.render(<" + displayName + " ... />)"],
            });
        }
    })
        .then(injectStyles(options))
        .then(function () {
        var _a, _b, _c;
        var document = cy.state('document');
        var reactDomToUse = options.ReactDom || react_dom_1.default;
        var el = document.getElementById(rootId);
        if (!el) {
            throw new Error([
                '[cypress-react-unit-test] 🔥 Hmm, cannot find root element to mount the component.',
                'Did you forget to include the support file?',
                'Check https://github.com/bahmutov/cypress-react-unit-test#install please',
            ].join(' '));
        }
        var key = 
        // @ts-ignore provide unique key to the the wrapped component to make sure we are rerendering between tests
        (((_c = (_b = (_a = Cypress === null || Cypress === void 0 ? void 0 : Cypress.mocha) === null || _a === void 0 ? void 0 : _a.getRunner()) === null || _b === void 0 ? void 0 : _b.test) === null || _c === void 0 ? void 0 : _c.title) || '') + Math.random();
        var props = {
            key: key,
        };
        var CypressTestComponent = reactDomToUse.render(React.createElement(React.Fragment, props, jsx), el);
        var logConsoleProps = {
            props: jsx.props,
        };
        if (logInstance) {
            logInstance.set('consoleProps', function () { return logConsoleProps; });
            if (el.children.length) {
                logInstance.set('$el', el.children.item(0));
            }
        }
        return cy
            .wrap(CypressTestComponent, { log: false })
            .as(options.alias || displayName)
            .then(function () {
            if (logInstance) {
                logInstance.snapshot('mounted');
                logInstance.end();
            }
            return undefined;
        });
    });
};
/**
 * Removes any mounted component
 */
exports.unmount = function () {
    checkMountModeEnabled();
    cy.log('unmounting...');
    var selector = '#' + rootId;
    return cy.get(selector, { log: false }).then(function ($el) {
        react_dom_1.unmountComponentAtNode($el[0]);
    });
};
exports.default = exports.mount;
